@@ -6,15 +6,15 @@ fz_display_list *
 fz_new_display_list_from_page(fz_context *ctx, fz_page *page)
 {
 	fz_display_list *list;
-	fz_rect bounds;
 	fz_device *dev = NULL;
 
-	list = fz_new_display_list(ctx, fz_bound_page(ctx, page, &bounds));
+	fz_var(dev);
 
+	list = fz_new_display_list(ctx, fz_bound_page(ctx, page));
 	fz_try(ctx)
 	{
 		dev = fz_new_list_device(ctx, list);
-		fz_run_page(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -50,15 +50,14 @@ fz_display_list *
 fz_new_display_list_from_page_contents(fz_context *ctx, fz_page *page)
 {
 	fz_display_list *list;
-	fz_rect bounds;
 	fz_device *dev = NULL;
 
-	list = fz_new_display_list(ctx, fz_bound_page(ctx, page, &bounds));
+	list = fz_new_display_list(ctx, fz_bound_page(ctx, page));
 
 	fz_try(ctx)
 	{
 		dev = fz_new_list_device(ctx, list);
-		fz_run_page_contents(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page_contents(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -78,15 +77,16 @@ fz_display_list *
 fz_new_display_list_from_annot(fz_context *ctx, fz_annot *annot)
 {
 	fz_display_list *list;
-	fz_rect bounds;
 	fz_device *dev = NULL;
 
-	list = fz_new_display_list(ctx, fz_bound_annot(ctx, annot, &bounds));
+	fz_var(dev);
+
+	list = fz_new_display_list(ctx, fz_bound_annot(ctx, annot));
 
 	fz_try(ctx)
 	{
 		dev = fz_new_list_device(ctx, list);
-		fz_run_annot(ctx, annot, dev, &fz_identity, NULL);
+		fz_run_annot(ctx, annot, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -103,18 +103,20 @@ fz_new_display_list_from_annot(fz_context *ctx, fz_annot *annot)
 }
 
 fz_pixmap *
-fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, fz_matrix ctm, fz_colorspace *cs, int alpha)
 {
 	fz_rect rect;
-	fz_irect irect;
+	fz_irect bbox;
 	fz_pixmap *pix;
 	fz_device *dev = NULL;
 
-	fz_bound_display_list(ctx, list, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
+	fz_var(dev);
 
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
+	rect = fz_bound_display_list(ctx, list);
+	rect = fz_transform_rect(rect, ctm);
+	bbox = fz_round_rect(rect);
+
+	pix = fz_new_pixmap_with_bbox(ctx, cs, bbox, 0, alpha);
 	if (alpha)
 		fz_clear_pixmap(ctx, pix);
 	else
@@ -123,7 +125,7 @@ fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, const fz
 	fz_try(ctx)
 	{
 		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_display_list(ctx, list, dev, &fz_identity, NULL, NULL);
+		fz_run_display_list(ctx, list, dev, fz_identity, fz_infinite_rect, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -140,18 +142,20 @@ fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, const fz
 }
 
 fz_pixmap *
-fz_new_pixmap_from_page_contents(fz_context *ctx, fz_page *page, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_page_contents(fz_context *ctx, fz_page *page, fz_matrix ctm, fz_colorspace *cs, int alpha)
 {
 	fz_rect rect;
-	fz_irect irect;
+	fz_irect bbox;
 	fz_pixmap *pix;
 	fz_device *dev = NULL;
 
-	fz_bound_page(ctx, page, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
+	fz_var(dev);
 
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
+	rect = fz_bound_page(ctx, page);
+	rect = fz_transform_rect(rect, ctm);
+	bbox = fz_round_rect(rect);
+
+	pix = fz_new_pixmap_with_bbox(ctx, cs, bbox, 0, alpha);
 	if (alpha)
 		fz_clear_pixmap(ctx, pix);
 	else
@@ -160,7 +164,7 @@ fz_new_pixmap_from_page_contents(fz_context *ctx, fz_page *page, const fz_matrix
 	fz_try(ctx)
 	{
 		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_page_contents(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page_contents(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -177,18 +181,20 @@ fz_new_pixmap_from_page_contents(fz_context *ctx, fz_page *page, const fz_matrix
 }
 
 fz_pixmap *
-fz_new_pixmap_from_annot(fz_context *ctx, fz_annot *annot, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_annot(fz_context *ctx, fz_annot *annot, fz_matrix ctm, fz_colorspace *cs, int alpha)
 {
 	fz_rect rect;
-	fz_irect irect;
+	fz_irect bbox;
 	fz_pixmap *pix;
 	fz_device *dev = NULL;
 
-	fz_bound_annot(ctx, annot, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
+	fz_var(dev);
 
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
+	rect = fz_bound_annot(ctx, annot);
+	rect = fz_transform_rect(rect, ctm);
+	bbox = fz_round_rect(rect);
+
+	pix = fz_new_pixmap_with_bbox(ctx, cs, bbox, 0, alpha);
 	if (alpha)
 		fz_clear_pixmap(ctx, pix);
 	else
@@ -197,7 +203,7 @@ fz_new_pixmap_from_annot(fz_context *ctx, fz_annot *annot, const fz_matrix *ctm,
 	fz_try(ctx)
 	{
 		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_annot(ctx, annot, dev, &fz_identity, NULL);
+		fz_run_annot(ctx, annot, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -214,27 +220,30 @@ fz_new_pixmap_from_annot(fz_context *ctx, fz_annot *annot, const fz_matrix *ctm,
 }
 
 fz_pixmap *
-fz_new_pixmap_from_page(fz_context *ctx, fz_page *page, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_page(fz_context *ctx, fz_page *page, fz_matrix ctm, fz_colorspace *cs, int alpha)
 {
 	fz_rect rect;
-	fz_irect irect;
+	fz_irect bbox;
 	fz_pixmap *pix;
 	fz_device *dev = NULL;
 
-	fz_bound_page(ctx, page, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
+	fz_var(dev);
 
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
-	if (alpha)
-		fz_clear_pixmap(ctx, pix);
-	else
-		fz_clear_pixmap_with_value(ctx, pix, 0xFF);
+	rect = fz_bound_page(ctx, page);
+	rect = fz_transform_rect(rect, ctm);
+	bbox = fz_round_rect(rect);
+
+	pix = fz_new_pixmap_with_bbox(ctx, cs, bbox, 0, alpha);
 
 	fz_try(ctx)
 	{
+		if (alpha)
+			fz_clear_pixmap(ctx, pix);
+		else
+			fz_clear_pixmap_with_value(ctx, pix, 0xFF);
+
 		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_page(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -251,7 +260,7 @@ fz_new_pixmap_from_page(fz_context *ctx, fz_page *page, const fz_matrix *ctm, fz
 }
 
 fz_pixmap *
-fz_new_pixmap_from_page_number(fz_context *ctx, fz_document *doc, int number, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_page_number(fz_context *ctx, fz_document *doc, int number, fz_matrix ctm, fz_colorspace *cs, int alpha)
 {
 	fz_page *page;
 	fz_pixmap *pix = NULL;
@@ -271,16 +280,17 @@ fz_new_stext_page_from_display_list(fz_context *ctx, fz_display_list *list, cons
 {
 	fz_stext_page *text;
 	fz_device *dev = NULL;
-	fz_rect mediabox;
+
+	fz_var(dev);
 
 	if (list == NULL)
 		return NULL;
 
-	text = fz_new_stext_page(ctx, fz_bound_display_list(ctx, list, &mediabox));
+	text = fz_new_stext_page(ctx, fz_bound_display_list(ctx, list));
 	fz_try(ctx)
 	{
 		dev = fz_new_stext_device(ctx, text, options);
-		fz_run_display_list(ctx, list, dev, &fz_identity, NULL, NULL);
+		fz_run_display_list(ctx, list, dev, fz_identity, fz_infinite_rect, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -301,16 +311,17 @@ fz_new_stext_page_from_page(fz_context *ctx, fz_page *page, const fz_stext_optio
 {
 	fz_stext_page *text;
 	fz_device *dev = NULL;
-	fz_rect mediabox;
+
+	fz_var(dev);
 
 	if (page == NULL)
 		return NULL;
 
-	text = fz_new_stext_page(ctx, fz_bound_page(ctx, page, &mediabox));
+	text = fz_new_stext_page(ctx, fz_bound_page(ctx, page));
 	fz_try(ctx)
 	{
 		dev = fz_new_stext_device(ctx, text, options);
-		fz_run_page_contents(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page_contents(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -342,8 +353,39 @@ fz_new_stext_page_from_page_number(fz_context *ctx, fz_document *doc, int number
 	return text;
 }
 
+fz_stext_page *
+fz_new_stext_page_from_annot(fz_context *ctx, fz_annot *annot, const fz_stext_options *options)
+{
+	fz_stext_page *text;
+	fz_device *dev = NULL;
+
+	fz_var(dev);
+
+	if (annot == NULL)
+		return NULL;
+
+	text = fz_new_stext_page(ctx, fz_bound_annot(ctx, annot));
+	fz_try(ctx)
+	{
+		dev = fz_new_stext_device(ctx, text, options);
+		fz_run_annot(ctx, annot, dev, fz_identity, NULL);
+		fz_close_device(ctx, dev);
+	}
+	fz_always(ctx)
+	{
+		fz_drop_device(ctx, dev);
+	}
+	fz_catch(ctx)
+	{
+		fz_drop_stext_page(ctx, text);
+		fz_rethrow(ctx);
+	}
+
+	return text;
+}
+
 int
-fz_search_display_list(fz_context *ctx, fz_display_list *list, const char *needle, fz_rect *hit_bbox, int hit_max)
+fz_search_display_list(fz_context *ctx, fz_display_list *list, const char *needle, fz_quad *hit_bbox, int hit_max)
 {
 	fz_stext_page *text;
 	int count = 0;
@@ -359,7 +401,7 @@ fz_search_display_list(fz_context *ctx, fz_display_list *list, const char *needl
 }
 
 int
-fz_search_page(fz_context *ctx, fz_page *page, const char *needle, fz_rect *hit_bbox, int hit_max)
+fz_search_page(fz_context *ctx, fz_page *page, const char *needle, fz_quad *hit_bbox, int hit_max)
 {
 	fz_stext_page *text;
 	int count = 0;
@@ -375,7 +417,7 @@ fz_search_page(fz_context *ctx, fz_page *page, const char *needle, fz_rect *hit_
 }
 
 int
-fz_search_page_number(fz_context *ctx, fz_document *doc, int number, const char *needle, fz_rect *hit_bbox, int hit_max)
+fz_search_page_number(fz_context *ctx, fz_document *doc, int number, const char *needle, fz_quad *hit_bbox, int hit_max)
 {
 	fz_page *page;
 	int count = 0;
