@@ -21,8 +21,8 @@ enum
 static int pdf_field_dirties_document(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 {
 	int ff = pdf_get_field_flags(ctx, doc, field);
-	if (ff & Ff_NoExport) return 0;
-	if (ff & Ff_ReadOnly) return 0;
+	if (ff & PDF_FIELD_IS_NO_EXPORT) return 0;
+	if (ff & PDF_FIELD_IS_READ_ONLY) return 0;
 	return 1;
 }
 
@@ -30,26 +30,22 @@ static int pdf_field_dirties_document(fz_context *ctx, pdf_document *doc, pdf_ob
  * share the same name */
 static pdf_obj *find_head_of_field_group(fz_context *ctx, pdf_obj *obj)
 {
-	if (obj == NULL || pdf_dict_get(ctx, obj, PDF_NAME_T))
+	if (obj == NULL || pdf_dict_get(ctx, obj, PDF_NAME(T)))
 		return obj;
 	else
-		return find_head_of_field_group(ctx, pdf_dict_get(ctx, obj, PDF_NAME_Parent));
+		return find_head_of_field_group(ctx, pdf_dict_get(ctx, obj, PDF_NAME(Parent)));
 }
 
 static void pdf_field_mark_dirty(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 {
-	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME_Kids);
+	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME(Kids));
 	if (kids)
 	{
 		int i, n = pdf_array_len(ctx, kids);
-
 		for (i = 0; i < n; i++)
 			pdf_field_mark_dirty(ctx, doc, pdf_array_get(ctx, kids, i));
 	}
-	else
-	{
-		pdf_dirty_obj(ctx, field);
-	}
+	pdf_dirty_obj(ctx, field);
 }
 
 static void update_field_value(fz_context *ctx, pdf_document *doc, pdf_obj *obj, const char *text)
@@ -65,27 +61,21 @@ static void update_field_value(fz_context *ctx, pdf_document *doc, pdf_obj *obj,
 	if (grp)
 		obj = grp;
 
-	pdf_dict_put_text_string(ctx, obj, PDF_NAME_V, text);
+	pdf_dict_put_text_string(ctx, obj, PDF_NAME(V), text);
 
 	pdf_field_mark_dirty(ctx, doc, obj);
 }
 
 static pdf_obj *find_field(fz_context *ctx, pdf_obj *dict, char *name, int len)
 {
-	pdf_obj *field;
-
 	int i, n = pdf_array_len(ctx, dict);
-
 	for (i = 0; i < n; i++)
 	{
-		char *part;
-
-		field = pdf_array_get(ctx, dict, i);
-		part = pdf_to_str_buf(ctx, pdf_dict_get(ctx, field, PDF_NAME_T));
+		pdf_obj *field = pdf_array_get(ctx, dict, i);
+		const char *part = pdf_dict_get_text_string(ctx, field, PDF_NAME(T));
 		if (strlen(part) == (size_t)len && !memcmp(part, name, len))
 			return field;
 	}
-
 	return NULL;
 }
 
@@ -108,7 +98,7 @@ pdf_obj *pdf_lookup_field(fz_context *ctx, pdf_obj *form, char *name)
 		len = dot ? dot - namep : (int)strlen(namep);
 		dict = find_field(ctx, form, namep, len);
 		if (dot)
-			form = pdf_dict_get(ctx, dict, PDF_NAME_Kids);
+			form = pdf_dict_get(ctx, dict, PDF_NAME(Kids));
 	}
 
 	return dict;
@@ -123,13 +113,13 @@ static void reset_form_field(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 	 * At the bottom of the hierarchy we may find widget annotations
 	 * that aren't also fields, but DV and V will not be present in their
 	 * dictionaries, and attempts to remove V will be harmless. */
-	pdf_obj *dv = pdf_dict_get(ctx, field, PDF_NAME_DV);
-	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME_Kids);
+	pdf_obj *dv = pdf_dict_get(ctx, field, PDF_NAME(DV));
+	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME(Kids));
 
 	if (dv)
-		pdf_dict_put(ctx, field, PDF_NAME_V, dv);
+		pdf_dict_put(ctx, field, PDF_NAME(V), dv);
 	else
-		pdf_dict_del(ctx, field, PDF_NAME_V);
+		pdf_dict_del(ctx, field, PDF_NAME(V));
 
 	if (kids == NULL)
 	{
@@ -142,14 +132,14 @@ static void reset_form_field(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 		case PDF_WIDGET_TYPE_RADIOBUTTON:
 		case PDF_WIDGET_TYPE_CHECKBOX:
 			{
-				pdf_obj *leafv = pdf_get_inheritable(ctx, doc, field, PDF_NAME_V);
+				pdf_obj *leafv = pdf_dict_get_inheritable(ctx, field, PDF_NAME(V));
 
 				if (leafv)
 					pdf_keep_obj(ctx, leafv);
 				else
-					leafv = PDF_NAME_Off;
+					leafv = PDF_NAME(Off);
 
-				pdf_dict_put_drop(ctx, field, PDF_NAME_AS, leafv);
+				pdf_dict_put_drop(ctx, field, PDF_NAME(AS), leafv);
 			}
 			break;
 
@@ -168,7 +158,7 @@ static void reset_form_field(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 
 void pdf_field_reset(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 {
-	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME_Kids);
+	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME(Kids));
 
 	reset_form_field(ctx, doc, field);
 
@@ -183,8 +173,8 @@ void pdf_field_reset(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 
 static void add_field_hierarchy_to_array(fz_context *ctx, pdf_obj *array, pdf_obj *field)
 {
-	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME_Kids);
-	pdf_obj *exclude = pdf_dict_get(ctx, field, PDF_NAME_Exclude);
+	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME(Kids));
+	pdf_obj *exclude = pdf_dict_get(ctx, field, PDF_NAME(Exclude));
 
 	if (exclude)
 		return;
@@ -209,12 +199,10 @@ static void add_field_hierarchy_to_array(fz_context *ctx, pdf_obj *array, pdf_ob
 */
 static pdf_obj *specified_fields(fz_context *ctx, pdf_document *doc, pdf_obj *fields, int exclude)
 {
-	pdf_obj *form = pdf_dict_getl(ctx, pdf_trailer(ctx, doc), PDF_NAME_Root, PDF_NAME_AcroForm, PDF_NAME_Fields, NULL);
+	pdf_obj *form = pdf_dict_getl(ctx, pdf_trailer(ctx, doc), PDF_NAME(Root), PDF_NAME(AcroForm), PDF_NAME(Fields), NULL);
 	int i, n;
 	pdf_obj *result = pdf_new_array(ctx, doc, 0);
-	pdf_obj *nil = NULL;
 
-	fz_var(nil);
 	fz_try(ctx)
 	{
 		/* The 'fields' array not being present signals that all fields
@@ -222,10 +210,7 @@ static pdf_obj *specified_fields(fz_context *ctx, pdf_document *doc, pdf_obj *fi
 		if (exclude || !fields)
 		{
 			/* mark the fields we don't want to act upon */
-			nil = pdf_new_null(ctx, doc);
-
 			n = pdf_array_len(ctx, fields);
-
 			for (i = 0; i < n; i++)
 			{
 				pdf_obj *field = pdf_array_get(ctx, fields, i);
@@ -234,7 +219,7 @@ static pdf_obj *specified_fields(fz_context *ctx, pdf_document *doc, pdf_obj *fi
 					field = pdf_lookup_field(ctx, form, pdf_to_str_buf(ctx, field));
 
 				if (field)
-					pdf_dict_put(ctx, field, PDF_NAME_Exclude, nil);
+					pdf_dict_put(ctx, field, PDF_NAME(Exclude), PDF_NULL);
 			}
 
 			/* Act upon all unmarked fields */
@@ -254,7 +239,7 @@ static pdf_obj *specified_fields(fz_context *ctx, pdf_document *doc, pdf_obj *fi
 					field = pdf_lookup_field(ctx, form, pdf_to_str_buf(ctx, field));
 
 				if (field)
-					pdf_dict_del(ctx, field, PDF_NAME_Exclude);
+					pdf_dict_del(ctx, field, PDF_NAME(Exclude));
 			}
 		}
 		else
@@ -273,10 +258,6 @@ static pdf_obj *specified_fields(fz_context *ctx, pdf_document *doc, pdf_obj *fi
 			}
 		}
 	}
-	fz_always(ctx)
-	{
-		pdf_drop_obj(ctx, nil);
-	}
 	fz_catch(ctx)
 	{
 		pdf_drop_obj(ctx, result);
@@ -307,15 +288,15 @@ static void reset_form(fz_context *ctx, pdf_document *doc, pdf_obj *fields, int
 	}
 }
 
-static void execute_action(fz_context *ctx, pdf_document *doc, pdf_obj *obj, pdf_obj *a)
+void pdf_execute_action(fz_context *ctx, pdf_document *doc, pdf_obj *obj, pdf_obj *a)
 {
 	if (a)
 	{
-		pdf_obj *type = pdf_dict_get(ctx, a, PDF_NAME_S);
+		pdf_obj *type = pdf_dict_get(ctx, a, PDF_NAME(S));
 
-		if (pdf_name_eq(ctx, type, PDF_NAME_JavaScript))
+		if (pdf_name_eq(ctx, type, PDF_NAME(JavaScript)))
 		{
-			pdf_obj *js = pdf_dict_get(ctx, a, PDF_NAME_JS);
+			pdf_obj *js = pdf_dict_get(ctx, a, PDF_NAME(JS));
 			if (js)
 			{
 				char *code = pdf_load_stream_or_string_as_utf8(ctx, js);
@@ -333,15 +314,15 @@ static void execute_action(fz_context *ctx, pdf_document *doc, pdf_obj *obj, pdf
 				}
 			}
 		}
-		else if (pdf_name_eq(ctx, type, PDF_NAME_ResetForm))
+		else if (pdf_name_eq(ctx, type, PDF_NAME(ResetForm)))
 		{
-			reset_form(ctx, doc, pdf_dict_get(ctx, a, PDF_NAME_Fields), pdf_to_int(ctx, pdf_dict_get(ctx, a, PDF_NAME_Flags)) & 1);
+			reset_form(ctx, doc, pdf_dict_get(ctx, a, PDF_NAME(Fields)), pdf_dict_get_int(ctx, a, PDF_NAME(Flags)) & 1);
 		}
-		else if (pdf_name_eq(ctx, type, PDF_NAME_Named))
+		else if (pdf_name_eq(ctx, type, PDF_NAME(Named)))
 		{
-			pdf_obj *name = pdf_dict_get(ctx, a, PDF_NAME_N);
+			pdf_obj *name = pdf_dict_get(ctx, a, PDF_NAME(N));
 
-			if (pdf_name_eq(ctx, name, PDF_NAME_Print))
+			if (pdf_name_eq(ctx, name, PDF_NAME(Print)))
 				pdf_event_issue_print(ctx, doc);
 		}
 	}
@@ -349,7 +330,7 @@ static void execute_action(fz_context *ctx, pdf_document *doc, pdf_obj *obj, pdf
 
 static void execute_action_chain(fz_context *ctx, pdf_document *doc, pdf_obj *obj)
 {
-	pdf_obj *a = pdf_dict_get(ctx, obj, PDF_NAME_A);
+	pdf_obj *a = pdf_dict_get(ctx, obj, PDF_NAME(A));
 	pdf_js_event e;
 
 	e.target = obj;
@@ -358,8 +339,8 @@ static void execute_action_chain(fz_context *ctx, pdf_document *doc, pdf_obj *ob
 
 	while (a)
 	{
-		execute_action(ctx, doc, obj, a);
-		a = pdf_dict_get(ctx, a, PDF_NAME_Next);
+		pdf_execute_action(ctx, doc, obj, a);
+		a = pdf_dict_get(ctx, a, PDF_NAME(Next));
 	}
 }
 
@@ -374,13 +355,13 @@ static void execute_additional_action(fz_context *ctx, pdf_document *doc, pdf_ob
 		e.target = obj;
 		e.value = "";
 		pdf_js_setup_event(doc->js, &e);
-		execute_action(ctx, doc, obj, a);
+		pdf_execute_action(ctx, doc, obj, a);
 	}
 }
 
 static void check_off(fz_context *ctx, pdf_document *doc, pdf_obj *obj)
 {
-	pdf_dict_put(ctx, obj, PDF_NAME_AS, PDF_NAME_Off);
+	pdf_dict_put(ctx, obj, PDF_NAME(AS), PDF_NAME(Off));
 }
 
 static void set_check(fz_context *ctx, pdf_document *doc, pdf_obj *chk, pdf_obj *name)
@@ -393,16 +374,16 @@ static void set_check(fz_context *ctx, pdf_document *doc, pdf_obj *chk, pdf_obj
 	if (pdf_dict_get(ctx, n, name))
 		val = name;
 	else
-		val = PDF_NAME_Off;
+		val = PDF_NAME(Off);
 
-	pdf_dict_put(ctx, chk, PDF_NAME_AS, val);
+	pdf_dict_put(ctx, chk, PDF_NAME(AS), val);
 }
 
 /* Set the values of all fields in a group defined by a node
  * in the hierarchy */
 static void set_check_grp(fz_context *ctx, pdf_document *doc, pdf_obj *grp, pdf_obj *val)
 {
-	pdf_obj *kids = pdf_dict_get(ctx, grp, PDF_NAME_Kids);
+	pdf_obj *kids = pdf_dict_get(ctx, grp, PDF_NAME(Kids));
 
 	if (kids == NULL)
 	{
@@ -419,10 +400,14 @@ static void set_check_grp(fz_context *ctx, pdf_document *doc, pdf_obj *grp, pdf_
 
 static void recalculate(fz_context *ctx, pdf_document *doc)
 {
+	pdf_js_event e = {NULL, NULL};
+
 	if (doc->recalculating)
 		return;
 
 	doc->recalculating = 1;
+
+	fz_var(e);
 	fz_try(ctx)
 	{
 		pdf_obj *co = pdf_dict_getp(ctx, pdf_trailer(ctx, doc), "Root/AcroForm/CO");
@@ -443,7 +428,10 @@ static void recalculate(fz_context *ctx, pdf_document *doc)
 					e.target = field;
 					e.value = pdf_field_value(ctx, doc, field);
 					pdf_js_setup_event(doc->js, &e);
-					execute_action(ctx, doc, field, calc);
+					/* e.value has been copied. We can free it */
+					fz_free(ctx, e.value);
+					e.value = NULL;
+					pdf_execute_action(ctx, doc, field, calc);
 					/* A calculate action, updates event.value. We need
 					* to place the value in the field */
 					update_field_value(ctx, doc, field, pdf_js_get_event(doc->js)->value);
@@ -453,6 +441,7 @@ static void recalculate(fz_context *ctx, pdf_document *doc)
 	}
 	fz_always(ctx)
 	{
+		fz_free(ctx, e.value);
 		doc->recalculating = 0;
 	}
 	fz_catch(ctx)
@@ -463,56 +452,57 @@ static void recalculate(fz_context *ctx, pdf_document *doc)
 
 static void toggle_check_box(fz_context *ctx, pdf_document *doc, pdf_obj *obj)
 {
-	pdf_obj *as = pdf_dict_get(ctx, obj, PDF_NAME_AS);
+	pdf_obj *as = pdf_dict_get(ctx, obj, PDF_NAME(AS));
 	int ff = pdf_get_field_flags(ctx, doc, obj);
-	int radio = ((ff & (Ff_Pushbutton|Ff_Radio)) == Ff_Radio);
-	char *val = NULL;
-	pdf_obj *grp = radio ? pdf_dict_get(ctx, obj, PDF_NAME_Parent) : find_head_of_field_group(ctx, obj);
+	int button_mask = PDF_BTN_FIELD_IS_RADIO | PDF_BTN_FIELD_IS_PUSHBUTTON;
+	int radio = ((ff & button_mask) == PDF_BTN_FIELD_IS_RADIO);
+	pdf_obj *val = NULL;
+	pdf_obj *grp = radio ? pdf_dict_get(ctx, obj, PDF_NAME(Parent)) : find_head_of_field_group(ctx, obj);
 
 	if (!grp)
 		grp = obj;
 
-	if (as && !pdf_name_eq(ctx, as, PDF_NAME_Off))
+	if (as && !pdf_name_eq(ctx, as, PDF_NAME(Off)))
 	{
-		/* "as" neither missing nor set to Off. Set it to Off, unless
+		/* AS neither missing nor set to Off. Set it to Off, unless
 		 * this is a non-toggle-off radio button. */
-		if ((ff & (Ff_Pushbutton|Ff_NoToggleToOff|Ff_Radio)) != (Ff_NoToggleToOff|Ff_Radio))
+		if (!(radio && (ff & PDF_BTN_FIELD_IS_NO_TOGGLE_TO_OFF)))
 		{
 			check_off(ctx, doc, obj);
-			val = "Off";
+			val = PDF_NAME(Off);
 		}
 	}
 	else
 	{
-		pdf_obj *n, *key = NULL;
+		pdf_obj *n, *key;
 		int len, i;
 
-		n = pdf_dict_getp(ctx, obj, "AP/N");
+		n = pdf_dict_getl(ctx, obj, PDF_NAME(AP), PDF_NAME(N), NULL);
 
-		/* Look for a key that isn't "Off" */
+		/* Look for an appearance state that isn't "Off" */
 		len = pdf_dict_len(ctx, n);
 		for (i = 0; i < len; i++)
 		{
 			key = pdf_dict_get_key(ctx, n, i);
-			if (pdf_is_name(ctx, key) && !pdf_name_eq(ctx, key, PDF_NAME_Off))
-				break;
+			if (pdf_is_name(ctx, key) && !pdf_name_eq(ctx, key, PDF_NAME(Off)))
+				val = key;
 		}
 
 		/* If we found no alternative value to Off then we have no value to use */
-		if (!key)
+		if (!val)
 			return;
 
 		if (radio)
 		{
 			/* For radio buttons, first turn off all buttons in the group and
 			 * then set the one that was clicked */
-			pdf_obj *kids = pdf_dict_get(ctx, grp, PDF_NAME_Kids);
+			pdf_obj *kids = pdf_dict_get(ctx, grp, PDF_NAME(Kids));
 
 			len = pdf_array_len(ctx, kids);
 			for (i = 0; i < len; i++)
 				check_off(ctx, doc, pdf_array_get(ctx, kids, i));
 
-			pdf_dict_put(ctx, obj, PDF_NAME_AS, key);
+			pdf_dict_put(ctx, obj, PDF_NAME(AS), val);
 		}
 		else
 		{
@@ -520,17 +510,13 @@ static void toggle_check_box(fz_context *ctx, pdf_document *doc, pdf_obj *obj)
 			 * below which all fields share a name with the clicked one. Set
 			 * all to the same value. This may cause the group to act like
 			 * radio buttons, if each have distinct "On" values */
-			if (grp)
-				set_check_grp(ctx, doc, grp, key);
-			else
-				set_check(ctx, doc, obj, key);
+			set_check_grp(ctx, doc, grp, val);
 		}
 	}
 
 	if (val && grp)
 	{
-		pdf_obj *v = pdf_new_text_string(ctx, doc, val);
-		pdf_dict_put_drop(ctx, grp, PDF_NAME_V, v);
+		pdf_dict_put(ctx, grp, PDF_NAME(V), val);
 		recalculate(ctx, doc);
 	}
 }
@@ -542,7 +528,8 @@ int pdf_has_unsaved_changes(fz_context *ctx, pdf_document *doc)
 
 int pdf_pass_event(fz_context *ctx, pdf_document *doc, pdf_page *page, pdf_ui_event *ui_event)
 {
-	pdf_annot *annot;
+	pdf_annot *a;
+	pdf_annot *annot = NULL;
 	pdf_hotspot *hp = &doc->hotspot;
 	fz_point *pt = &(ui_event->event.pointer.pt);
 	int changed = 0;
@@ -551,26 +538,27 @@ int pdf_pass_event(fz_context *ctx, pdf_document *doc, pdf_page *page, pdf_ui_ev
 	if (page == NULL)
 		return 0;
 
-	for (annot = page->annots; annot; annot = annot->next)
+	for (a = page->annots; a; a = a->next)
 	{
-		pdf_bound_annot(ctx, annot, &bbox);
+		bbox = pdf_bound_annot(ctx, a);
 		if (pt->x >= bbox.x0 && pt->x <= bbox.x1)
 			if (pt->y >= bbox.y0 && pt->y <= bbox.y1)
-				break;
+				annot = a;
 	}
 
-	/* Skip hidden annotations. */
+	/* Skip hidden annotations and read-only widgets. */
 	if (annot)
 	{
-		int f = pdf_to_int(ctx, pdf_dict_get(ctx, annot->obj, PDF_NAME_F));
-		if (f & (PDF_ANNOT_IS_HIDDEN|PDF_ANNOT_IS_NO_VIEW))
+		int ff = pdf_dict_get_int(ctx, annot->obj, PDF_NAME(Ff));
+		int f = pdf_dict_get_int(ctx, annot->obj, PDF_NAME(F));
+		if (f & (PDF_ANNOT_IS_HIDDEN|PDF_ANNOT_IS_NO_VIEW) || ff & PDF_FIELD_IS_READ_ONLY)
 			annot = NULL;
 	}
 
 	/* Skip Link annotations. */
 	if (annot)
 	{
-		if (pdf_name_eq(ctx, pdf_dict_get(ctx, annot->obj, PDF_NAME_Subtype), PDF_NAME_Link))
+		if (pdf_name_eq(ctx, pdf_dict_get(ctx, annot->obj, PDF_NAME(Subtype)), PDF_NAME(Link)))
 			annot = NULL;
 	}
 
@@ -637,14 +625,17 @@ int pdf_pass_event(fz_context *ctx, pdf_document *doc, pdf_page *page, pdf_ui_ev
 	return changed;
 }
 
-void pdf_update_page(fz_context *ctx, pdf_page *page)
+int
+pdf_update_page(fz_context *ctx, pdf_page *page)
 {
 	pdf_annot *annot;
-
+	int changed = 0;
 	for (annot = page->annots; annot; annot = annot->next)
 	{
-		pdf_update_annot(ctx, annot);
+		if (pdf_update_annot(ctx, annot))
+			changed = 1;
 	}
+	return changed;
 }
 
 pdf_widget *pdf_focused_widget(fz_context *ctx, pdf_document *doc)
@@ -684,12 +675,12 @@ pdf_widget *pdf_create_widget(fz_context *ctx, pdf_document *doc, pdf_page *page
 	fz_try(ctx)
 	{
 		pdf_set_field_type(ctx, doc, annot->obj, type);
-		pdf_dict_put_string(ctx, annot->obj, PDF_NAME_T, fieldname, strlen(fieldname));
+		pdf_dict_put_text_string(ctx, annot->obj, PDF_NAME(T), fieldname);
 
 		if (type == PDF_WIDGET_TYPE_SIGNATURE)
 		{
 			int sigflags = (old_sigflags | (SigFlag_SignaturesExist|SigFlag_AppendOnly));
-			pdf_dict_putl_drop(ctx, pdf_trailer(ctx, doc), pdf_new_int(ctx, doc, sigflags), PDF_NAME_Root, PDF_NAME_AcroForm, PDF_NAME_SigFlags, NULL);
+			pdf_dict_putl_drop(ctx, pdf_trailer(ctx, doc), pdf_new_int(ctx, sigflags), PDF_NAME(Root), PDF_NAME(AcroForm), PDF_NAME(SigFlags), NULL);
 		}
 
 		/*
@@ -700,7 +691,7 @@ pdf_widget *pdf_create_widget(fz_context *ctx, pdf_document *doc, pdf_page *page
 		if (!form)
 		{
 			form = pdf_new_array(ctx, doc, 1);
-			pdf_dict_putl_drop(ctx, pdf_trailer(ctx, doc), form, PDF_NAME_Root, PDF_NAME_AcroForm, PDF_NAME_Fields, NULL);
+			pdf_dict_putl_drop(ctx, pdf_trailer(ctx, doc), form, PDF_NAME(Root), PDF_NAME(AcroForm), PDF_NAME(Fields), NULL);
 		}
 
 		pdf_array_push(ctx, form, annot->obj); /* Cleanup relies on this statement being last */
@@ -712,7 +703,7 @@ pdf_widget *pdf_create_widget(fz_context *ctx, pdf_document *doc, pdf_page *page
 		/* An empty Fields array may have been created, but that is harmless */
 
 		if (type == PDF_WIDGET_TYPE_SIGNATURE)
-			pdf_dict_putl_drop(ctx, pdf_trailer(ctx, doc), pdf_new_int(ctx, doc, old_sigflags), PDF_NAME_Root, PDF_NAME_AcroForm, PDF_NAME_SigFlags, NULL);
+			pdf_dict_putl_drop(ctx, pdf_trailer(ctx, doc), pdf_new_int(ctx, old_sigflags), PDF_NAME(Root), PDF_NAME(AcroForm), PDF_NAME(SigFlags), NULL);
 
 		fz_rethrow(ctx);
 	}
@@ -735,15 +726,12 @@ static int set_text_field_value(fz_context *ctx, pdf_document *doc, pdf_obj *fie
 	if (v && doc->js)
 	{
 		pdf_js_event e;
-
 		e.target = field;
 		e.value = fz_strdup(ctx, text);
 		pdf_js_setup_event(doc->js, &e);
-		execute_action(ctx, doc, field, v);
-
+		pdf_execute_action(ctx, doc, field, v);
 		if (!pdf_js_get_event(doc->js)->rc)
 			return 0;
-
 		text = pdf_js_get_event(doc->js)->value;
 	}
 
@@ -756,7 +744,7 @@ static int set_text_field_value(fz_context *ctx, pdf_document *doc, pdf_obj *fie
 
 static void update_checkbox_selector(fz_context *ctx, pdf_document *doc, pdf_obj *field, const char *val)
 {
-	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME_Kids);
+	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME(Kids));
 
 	if (kids)
 	{
@@ -771,10 +759,10 @@ static void update_checkbox_selector(fz_context *ctx, pdf_document *doc, pdf_obj
 		pdf_obj *oval;
 
 		if (pdf_dict_gets(ctx, n, val))
-			oval = pdf_new_name(ctx, doc, val);
+			oval = pdf_new_name(ctx, val);
 		else
-			oval = PDF_NAME_Off;
-		pdf_dict_put_drop(ctx, field, PDF_NAME_AS, oval);
+			oval = PDF_NAME(Off);
+		pdf_dict_put_drop(ctx, field, PDF_NAME(AS), oval);
 	}
 }
 
@@ -814,7 +802,7 @@ int pdf_field_set_value(fz_context *ctx, pdf_document *doc, pdf_obj *field, cons
 
 char *pdf_field_border_style(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 {
-	const char *bs = pdf_to_name(ctx, pdf_dict_getl(ctx, field, PDF_NAME_BS, PDF_NAME_S, NULL));
+	const char *bs = pdf_to_name(ctx, pdf_dict_getl(ctx, field, PDF_NAME(BS), PDF_NAME(S), NULL));
 	switch (*bs)
 	{
 	case 'S': return "Solid";
@@ -831,19 +819,19 @@ void pdf_field_set_border_style(fz_context *ctx, pdf_document *doc, pdf_obj *fie
 	pdf_obj *val;
 
 	if (!strcmp(text, "Solid"))
-		val = PDF_NAME_S;
+		val = PDF_NAME(S);
 	else if (!strcmp(text, "Dashed"))
-		val = PDF_NAME_D;
+		val = PDF_NAME(D);
 	else if (!strcmp(text, "Beveled"))
-		val = PDF_NAME_B;
+		val = PDF_NAME(B);
 	else if (!strcmp(text, "Inset"))
-		val = PDF_NAME_I;
+		val = PDF_NAME(I);
 	else if (!strcmp(text, "Underline"))
-		val = PDF_NAME_U;
+		val = PDF_NAME(U);
 	else
 		return;
 
-	pdf_dict_putl_drop(ctx, field, val, PDF_NAME_BS, PDF_NAME_S, NULL);
+	pdf_dict_putl_drop(ctx, field, val, PDF_NAME(BS), PDF_NAME(S), NULL);
 	pdf_field_mark_dirty(ctx, doc, field);
 }
 
@@ -851,8 +839,8 @@ void pdf_field_set_button_caption(fz_context *ctx, pdf_document *doc, pdf_obj *f
 {
 	if (pdf_field_type(ctx, doc, field) == PDF_WIDGET_TYPE_PUSHBUTTON)
 	{
-		pdf_obj *val = pdf_new_text_string(ctx, doc, text);
-		pdf_dict_putl_drop(ctx, field, val, PDF_NAME_MK, PDF_NAME_CA, NULL);
+		pdf_obj *val = pdf_new_text_string(ctx, text);
+		pdf_dict_putl_drop(ctx, field, val, PDF_NAME(MK), PDF_NAME(CA), NULL);
 		pdf_field_mark_dirty(ctx, doc, field);
 	}
 }
@@ -865,10 +853,10 @@ int pdf_field_display(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 	/* Base response on first of children. Not ideal,
 	 * but not clear how to handle children with
 	 * differing values */
-	while ((kids = pdf_dict_get(ctx, field, PDF_NAME_Kids)) != NULL)
+	while ((kids = pdf_dict_get(ctx, field, PDF_NAME(Kids))) != NULL)
 		field = pdf_array_get(ctx, kids, 0);
 
-	f = pdf_to_int(ctx, pdf_dict_get(ctx, field, PDF_NAME_F));
+	f = pdf_dict_get_int(ctx, field, PDF_NAME(F));
 
 	if (f & PDF_ANNOT_IS_HIDDEN)
 	{
@@ -897,8 +885,8 @@ int pdf_field_display(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 static char *get_field_name(fz_context *ctx, pdf_document *doc, pdf_obj *field, int spare)
 {
 	char *res = NULL;
-	pdf_obj *parent = pdf_dict_get(ctx, field, PDF_NAME_Parent);
-	char *lname = pdf_to_str_buf(ctx, pdf_dict_get(ctx, field, PDF_NAME_T));
+	pdf_obj *parent = pdf_dict_get(ctx, field, PDF_NAME(Parent));
+	const char *lname = pdf_dict_get_text_string(ctx, field, PDF_NAME(T));
 	int llen = (int)strlen(lname);
 
 	/*
@@ -934,14 +922,24 @@ char *pdf_field_name(fz_context *ctx, pdf_document *doc, pdf_obj *field)
 	return get_field_name(ctx, doc, field, 0);
 }
 
+const char *pdf_field_label(fz_context *ctx, pdf_document *doc, pdf_obj *field)
+{
+	pdf_obj *label = pdf_dict_get_inheritable(ctx, field, PDF_NAME(TU));
+	if (!label)
+		label = pdf_dict_get_inheritable(ctx, field, PDF_NAME(T));
+	if (label)
+		return pdf_to_text_string(ctx, label);
+	return "Text Field";
+}
+
 void pdf_field_set_display(fz_context *ctx, pdf_document *doc, pdf_obj *field, int d)
 {
-	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME_Kids);
+	pdf_obj *kids = pdf_dict_get(ctx, field, PDF_NAME(Kids));
 
 	if (!kids)
 	{
 		int mask = (PDF_ANNOT_IS_HIDDEN|PDF_ANNOT_IS_PRINT|PDF_ANNOT_IS_NO_VIEW);
-		int f = pdf_to_int(ctx, pdf_dict_get(ctx, field, PDF_NAME_F)) & ~mask;
+		int f = pdf_dict_get_int(ctx, field, PDF_NAME(F)) & ~mask;
 		pdf_obj *fo;
 
 		switch (d)
@@ -959,8 +957,8 @@ void pdf_field_set_display(fz_context *ctx, pdf_document *doc, pdf_obj *field, i
 			break;
 		}
 
-		fo = pdf_new_int(ctx, doc, f);
-		pdf_dict_put_drop(ctx, field, PDF_NAME_F, fo);
+		fo = pdf_new_int(ctx, f);
+		pdf_dict_put_drop(ctx, field, PDF_NAME(F), fo);
 	}
 	else
 	{
@@ -976,56 +974,48 @@ void pdf_field_set_fill_color(fz_context *ctx, pdf_document *doc, pdf_obj *field
 	/* col == NULL mean transparent, but we can simply pass it on as with
 	 * non-NULL values because pdf_dict_putp interprets a NULL value as
 	 * delete */
-	pdf_dict_putl(ctx, field, col, PDF_NAME_MK, PDF_NAME_BG, NULL);
+	pdf_dict_putl(ctx, field, col, PDF_NAME(MK), PDF_NAME(BG), NULL);
 	pdf_field_mark_dirty(ctx, doc, field);
 }
 
 void pdf_field_set_text_color(fz_context *ctx, pdf_document *doc, pdf_obj *field, pdf_obj *col)
 {
-	pdf_da_info di;
-	fz_buffer *fzbuf = NULL;
-	char *da = pdf_to_str_buf(ctx, pdf_get_inheritable(ctx, doc, field, PDF_NAME_DA));
-	unsigned char *buf;
-	int ilen;
-	size_t len;
-	pdf_obj *daobj;
+	char buf[100];
+	const char *font;
+	float size, color[3], black;
+	const char *da = pdf_to_str_buf(ctx, pdf_dict_get_inheritable(ctx, field, PDF_NAME(DA)));
 
-	memset(&di, 0, sizeof(di));
+	pdf_parse_default_appearance(ctx, da, &font, &size, color);
 
-	fz_var(fzbuf);
-	fz_var(di);
-	fz_try(ctx)
+	switch (pdf_array_len(ctx, col))
 	{
-		int i;
-
-		pdf_parse_da(ctx, da, &di);
-		di.col_size = pdf_array_len(ctx, col);
-
-		ilen = fz_mini(di.col_size, (int)nelem(di.col));
-		for (i = 0; i < ilen; i++)
-			di.col[i] = pdf_to_real(ctx, pdf_array_get(ctx, col, i));
-
-		fzbuf = fz_new_buffer(ctx, 0);
-		pdf_fzbuf_print_da(ctx, fzbuf, &di);
-		len = fz_buffer_storage(ctx, fzbuf, &buf);
-		daobj = pdf_new_string(ctx, doc, (char *)buf, len);
-		pdf_dict_put_drop(ctx, field, PDF_NAME_DA, daobj);
-		pdf_field_mark_dirty(ctx, doc, field);
-	}
-	fz_always(ctx)
-	{
-		pdf_da_info_fin(ctx, &di);
-		fz_drop_buffer(ctx, fzbuf);
-	}
-	fz_catch(ctx)
-	{
-		fz_warn(ctx, "%s", fz_caught_message(ctx));
+	default:
+		color[0] = color[1] = color[2] = 0;
+		break;
+	case 1:
+		color[0] = color[1] = color[2] = pdf_array_get_real(ctx, col, 0);
+		break;
+	case 3:
+		color[0] = pdf_array_get_real(ctx, col, 0);
+		color[1] = pdf_array_get_real(ctx, col, 1);
+		color[2] = pdf_array_get_real(ctx, col, 2);
+		break;
+	case 4:
+		black = pdf_array_get_real(ctx, col, 3);
+		color[0] = 1 - fz_min(1, pdf_array_get_real(ctx, col, 0) + black);
+		color[1] = 1 - fz_min(1, pdf_array_get_real(ctx, col, 1) + black);
+		color[2] = 1 - fz_min(1, pdf_array_get_real(ctx, col, 2) + black);
+		break;
 	}
+
+	pdf_print_default_appearance(ctx, buf, sizeof buf, font, size, color);
+	pdf_dict_put_string(ctx, field, PDF_NAME(DA), buf, strlen(buf));
+	pdf_field_mark_dirty(ctx, doc, field);
 }
 
-fz_rect *pdf_bound_widget(fz_context *ctx, pdf_widget *widget, fz_rect *rect)
+fz_rect pdf_bound_widget(fz_context *ctx, pdf_widget *widget)
 {
-	return pdf_bound_annot(ctx, (pdf_annot*)widget, rect);
+	return pdf_bound_annot(ctx, (pdf_annot*)widget);
 }
 
 char *pdf_text_widget_text(fz_context *ctx, pdf_document *doc, pdf_widget *tw)
@@ -1050,46 +1040,38 @@ int pdf_text_widget_max_len(fz_context *ctx, pdf_document *doc, pdf_widget *tw)
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 
-	return pdf_to_int(ctx, pdf_get_inheritable(ctx, doc, annot->obj, PDF_NAME_MaxLen));
+	return pdf_to_int(ctx, pdf_dict_get_inheritable(ctx, annot->obj, PDF_NAME(MaxLen)));
 }
 
 int pdf_text_widget_content_type(fz_context *ctx, pdf_document *doc, pdf_widget *tw)
 {
 	pdf_annot *annot = (pdf_annot *)tw;
-	char *code = NULL;
 	int type = PDF_WIDGET_CONTENT_UNRESTRAINED;
-
-	fz_var(code);
-	fz_try(ctx)
-	{
-		code = pdf_get_string_or_stream(ctx, doc, pdf_dict_getl(ctx, annot->obj, PDF_NAME_AA, PDF_NAME_F, PDF_NAME_JS, NULL));
-		if (code)
-		{
-			if (strstr(code, "AFNumber_Format"))
-				type = PDF_WIDGET_CONTENT_NUMBER;
-			else if (strstr(code, "AFSpecial_Format"))
-				type = PDF_WIDGET_CONTENT_SPECIAL;
-			else if (strstr(code, "AFDate_FormatEx"))
-				type = PDF_WIDGET_CONTENT_DATE;
-			else if (strstr(code, "AFTime_FormatEx"))
-				type = PDF_WIDGET_CONTENT_TIME;
-		}
-	}
-	fz_always(ctx)
+	pdf_obj *js = pdf_dict_getl(ctx, annot->obj, PDF_NAME(AA), PDF_NAME(F), PDF_NAME(JS), NULL);
+	if (js)
 	{
+		char *code = pdf_load_stream_or_string_as_utf8(ctx, js);
+		if (strstr(code, "AFNumber_Format"))
+			type = PDF_WIDGET_CONTENT_NUMBER;
+		else if (strstr(code, "AFSpecial_Format"))
+			type = PDF_WIDGET_CONTENT_SPECIAL;
+		else if (strstr(code, "AFDate_FormatEx"))
+			type = PDF_WIDGET_CONTENT_DATE;
+		else if (strstr(code, "AFTime_FormatEx"))
+			type = PDF_WIDGET_CONTENT_TIME;
 		fz_free(ctx, code);
 	}
-	fz_catch(ctx)
-	{
-		fz_warn(ctx, "failure in fz_text_widget_content_type");
-	}
 
 	return type;
 }
 
 static int run_keystroke(fz_context *ctx, pdf_document *doc, pdf_obj *field, char **text)
 {
-	pdf_obj *k = pdf_dict_getl(ctx, field, PDF_NAME_AA, PDF_NAME_K, NULL);
+	pdf_obj *k = pdf_dict_getl(ctx, field, PDF_NAME(AA), PDF_NAME(K), NULL);
+
+	/* Return 1 on empty string */
+	if (*text[0] == 0)
+		return 1;
 
 	if (k && doc->js)
 	{
@@ -1098,7 +1080,7 @@ static int run_keystroke(fz_context *ctx, pdf_document *doc, pdf_obj *field, cha
 		e.target = field;
 		e.value = *text;
 		pdf_js_setup_event(doc->js, &e);
-		execute_action(ctx, doc, field, k);
+		pdf_execute_action(ctx, doc, field, k);
 
 		if (!pdf_js_get_event(doc->js)->rc)
 			return 0;
@@ -1129,7 +1111,7 @@ int pdf_text_widget_set_text(fz_context *ctx, pdf_document *doc, pdf_widget *tw,
 }
 
 /* Get either the listed value or the export value. */
-int pdf_choice_widget_options(fz_context *ctx, pdf_document *doc, pdf_widget *tw, int exportval, char *opts[])
+int pdf_choice_widget_options(fz_context *ctx, pdf_document *doc, pdf_widget *tw, int exportval, const char *opts[])
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 	pdf_obj *optarr;
@@ -1138,7 +1120,7 @@ int pdf_choice_widget_options(fz_context *ctx, pdf_document *doc, pdf_widget *tw
 	if (!annot)
 		return 0;
 
-	optarr = pdf_dict_get(ctx, annot->obj, PDF_NAME_Opt);
+	optarr = pdf_dict_get(ctx, annot->obj, PDF_NAME(Opt));
 	n = pdf_array_len(ctx, optarr);
 
 	if (opts)
@@ -1149,11 +1131,11 @@ int pdf_choice_widget_options(fz_context *ctx, pdf_document *doc, pdf_widget *tw
 			/* If it is a two element array, the second item is the one that we want if we want the listing value. */
 			if (m == 2)
 				if (exportval)
-					opts[i] = pdf_to_utf8(ctx, pdf_array_get(ctx, pdf_array_get(ctx, optarr, i), 0));
+					opts[i] = pdf_array_get_text_string(ctx, pdf_array_get(ctx, optarr, i), 0);
 				else
-					opts[i] = pdf_to_utf8(ctx, pdf_array_get(ctx, pdf_array_get(ctx, optarr, i), 1));
+					opts[i] = pdf_array_get_text_string(ctx, pdf_array_get(ctx, optarr, i), 1);
 			else
-				opts[i] = pdf_to_utf8(ctx, pdf_array_get(ctx, optarr, i));
+				opts[i] = pdf_array_get_text_string(ctx, optarr, i);
 		}
 	}
 	return n;
@@ -1169,13 +1151,13 @@ int pdf_choice_widget_is_multiselect(fz_context *ctx, pdf_document *doc, pdf_wid
 	{
 	case PDF_WIDGET_TYPE_LISTBOX:
 	case PDF_WIDGET_TYPE_COMBOBOX:
-		return (pdf_get_field_flags(ctx, doc, annot->obj) & Ff_MultiSelect) != 0;
+		return (pdf_get_field_flags(ctx, doc, annot->obj) & PDF_CH_FIELD_IS_MULTI_SELECT) != 0;
 	default:
 		return 0;
 	}
 }
 
-int pdf_choice_widget_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw, char *opts[])
+int pdf_choice_widget_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw, const char *opts[])
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 	pdf_obj *optarr;
@@ -1184,37 +1166,32 @@ int pdf_choice_widget_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw,
 	if (!annot)
 		return 0;
 
-	optarr = pdf_dict_get(ctx, annot->obj, PDF_NAME_V);
+	optarr = pdf_dict_get(ctx, annot->obj, PDF_NAME(V));
 
 	if (pdf_is_string(ctx, optarr))
 	{
 		if (opts)
-			opts[0] = pdf_to_utf8(ctx, optarr);
-
+			opts[0] = pdf_to_text_string(ctx, optarr);
 		return 1;
 	}
 	else
 	{
 		n = pdf_array_len(ctx, optarr);
-
 		if (opts)
 		{
 			for (i = 0; i < n; i++)
 			{
 				pdf_obj *elem = pdf_array_get(ctx, optarr, i);
-
 				if (pdf_is_array(ctx, elem))
 					elem = pdf_array_get(ctx, elem, 1);
-
-				opts[i] = pdf_to_utf8(ctx, elem);
+				opts[i] = pdf_to_text_string(ctx, elem);
 			}
 		}
-
 		return n;
 	}
 }
 
-void pdf_choice_widget_set_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw, int n, char *opts[])
+void pdf_choice_widget_set_value(fz_context *ctx, pdf_document *doc, pdf_widget *tw, int n, const char *opts[])
 {
 	pdf_annot *annot = (pdf_annot *)tw;
 	pdf_obj *optarr = NULL, *opt;
@@ -1232,20 +1209,20 @@ void pdf_choice_widget_set_value(fz_context *ctx, pdf_document *doc, pdf_widget
 
 			for (i = 0; i < n; i++)
 			{
-				opt = pdf_new_text_string(ctx, doc, opts[i]);
+				opt = pdf_new_text_string(ctx, opts[i]);
 				pdf_array_push_drop(ctx, optarr, opt);
 			}
 
-			pdf_dict_put_drop(ctx, annot->obj, PDF_NAME_V, optarr);
+			pdf_dict_put_drop(ctx, annot->obj, PDF_NAME(V), optarr);
 		}
 		else
 		{
-			opt = pdf_new_text_string(ctx, doc, opts[0]);
-			pdf_dict_put_drop(ctx, annot->obj, PDF_NAME_V, opt);
+			opt = pdf_new_text_string(ctx, opts[0]);
+			pdf_dict_put_drop(ctx, annot->obj, PDF_NAME(V), opt);
 		}
 
 		/* FIXME: when n > 1, we should be regenerating the indexes */
-		pdf_dict_del(ctx, annot->obj, PDF_NAME_I);
+		pdf_dict_del(ctx, annot->obj, PDF_NAME(I));
 
 		pdf_field_mark_dirty(ctx, doc, annot->obj);
 		if (pdf_field_dirties_document(ctx, doc, annot->obj))
@@ -1261,15 +1238,15 @@ void pdf_choice_widget_set_value(fz_context *ctx, pdf_document *doc, pdf_widget
 int pdf_signature_widget_byte_range(fz_context *ctx, pdf_document *doc, pdf_widget *widget, fz_range *byte_range)
 {
 	pdf_annot *annot = (pdf_annot *)widget;
-	pdf_obj *br = pdf_dict_getl(ctx, annot->obj, PDF_NAME_V, PDF_NAME_ByteRange, NULL);
+	pdf_obj *br = pdf_dict_getl(ctx, annot->obj, PDF_NAME(V), PDF_NAME(ByteRange), NULL);
 	int i, n = pdf_array_len(ctx, br)/2;
 
 	if (byte_range)
 	{
 		for (i = 0; i < n; i++)
 		{
-			byte_range[i].offset = pdf_to_int(ctx, pdf_array_get(ctx, br, 2*i));
-			byte_range[i].len = pdf_to_int(ctx, pdf_array_get(ctx, br, 2*i+1));
+			byte_range[i].offset = pdf_array_get_int(ctx, br, 2*i);
+			byte_range[i].length = pdf_array_get_int(ctx, br, 2*i+1);
 		}
 	}
 
@@ -1292,7 +1269,7 @@ fz_stream *pdf_signature_widget_hash_bytes(fz_context *ctx, pdf_document *doc, p
 			pdf_signature_widget_byte_range(ctx, doc, widget, byte_range);
 		}
 
-		bytes = fz_open_null_n(ctx, doc->file, byte_range, byte_range_len);
+		bytes = fz_open_range_filter(ctx, doc->file, byte_range, byte_range_len);
 	}
 	fz_always(ctx)
 	{
@@ -1309,7 +1286,7 @@ fz_stream *pdf_signature_widget_hash_bytes(fz_context *ctx, pdf_document *doc, p
 int pdf_signature_widget_contents(fz_context *ctx, pdf_document *doc, pdf_widget *widget, char **contents)
 {
 	pdf_annot *annot = (pdf_annot *)widget;
-	pdf_obj *c = pdf_dict_getl(ctx, annot->obj, PDF_NAME_V, PDF_NAME_Contents, NULL);
+	pdf_obj *c = pdf_dict_getl(ctx, annot->obj, PDF_NAME(V), PDF_NAME(Contents), NULL);
 	if (contents)
 		*contents = pdf_to_str_buf(ctx, c);
 	return pdf_to_str_len(ctx, c);
@@ -1322,40 +1299,45 @@ void pdf_signature_set_value(fz_context *ctx, pdf_document *doc, pdf_obj *field,
 	int vnum;
 	pdf_obj *byte_range;
 	pdf_obj *contents;
-	char buf[2048];
-
-	memset(buf, 0, sizeof(buf));
+	int max_digest_size;
+	char *buf = NULL;
 
 	vnum = pdf_create_object(ctx, doc);
 	indv = pdf_new_indirect(ctx, doc, vnum, 0);
-	pdf_dict_put_drop(ctx, field, PDF_NAME_V, indv);
+	pdf_dict_put_drop(ctx, field, PDF_NAME(V), indv);
+
+	max_digest_size = signer->max_digest_size(signer);
 
 	fz_var(v);
+	fz_var(buf);
 	fz_try(ctx)
 	{
 		v = pdf_new_dict(ctx, doc, 4);
 		pdf_update_object(ctx, doc, vnum, v);
+
+		buf = fz_calloc(ctx, max_digest_size, 1);
+
+		byte_range = pdf_new_array(ctx, doc, 4);
+		pdf_dict_put_drop(ctx, v, PDF_NAME(ByteRange), byte_range);
+
+		contents = pdf_new_string(ctx, buf, max_digest_size);
+		pdf_dict_put_drop(ctx, v, PDF_NAME(Contents), contents);
+
+		pdf_dict_put(ctx, v, PDF_NAME(Filter), PDF_NAME(Adobe_PPKLite));
+		pdf_dict_put(ctx, v, PDF_NAME(SubFilter), PDF_NAME(adbe_pkcs7_detached));
+
+		/* Record details within the document structure so that contents
+		* and byte_range can be updated with their correct values at
+		* saving time */
+		pdf_xref_store_unsaved_signature(ctx, doc, field, signer);
 	}
 	fz_always(ctx)
 	{
 		pdf_drop_obj(ctx, v);
+		fz_free(ctx, buf);
 	}
 	fz_catch(ctx)
 	{
 		fz_rethrow(ctx);
 	}
-
-	byte_range = pdf_new_array(ctx, doc, 4);
-	pdf_dict_put_drop(ctx, v, PDF_NAME_ByteRange, byte_range);
-
-	contents = pdf_new_string(ctx, doc, buf, sizeof(buf));
-	pdf_dict_put_drop(ctx, v, PDF_NAME_Contents, contents);
-
-	pdf_dict_put(ctx, v, PDF_NAME_Filter, PDF_NAME_Adobe_PPKLite);
-	pdf_dict_put(ctx, v, PDF_NAME_SubFilter, PDF_NAME_adbe_pkcs7_detached);
-
-	/* Record details within the document structure so that contents
-	 * and byte_range can be updated with their correct values at
-	 * saving time */
-	pdf_xref_store_unsaved_signature(ctx, doc, field, signer);
 }
